from matrix import *
from formula import *
import math

def magnitude(a: list) -> float:
    return pyth(pyth(a[0][0], a[0][1]), a[0][2])
    
def dot(a: list, b: list) -> float:
    if(a == b):
        return magnitude(a) ** 2
    else:
        bCol = []
        for i in range(len(b[0])):
            bCol.append([b[0][i]])
        return multiplication(a, bCol)[0][0]

def angle(a: list, b: list, mode: bool) -> float:
    if(mode):
        return math.acos(dot(a, b) / (magnitude(a) * magnitude(b)))
    else:
        return math.acos(dot(a, b) / (magnitude(a) * magnitude(b))) / math.pi * 180

def cross(a: list, b: list) -> list:
    if(a == b):
        return [[0, 0, 0]]
    else:
        crossVector = []
        crossMatrix = [[1, 1, 1], a[0], b[0]]
        for i in range(3):
            crossVector.append(getCofactor(crossMatrix,0, i))
        return crossVector

def projection(a: list, b: list) -> list:
    scalar = dot(a, b) / (magnitude(b) ** 2)
    projectionVector = []
    for i in range(3):
        projectionVector.append(b[0][i] * scalar)
    return projectionVector


"""
format: list(list(int))
"""

def add(a: list, b: list) -> list:
    if(len(a) != len(b) or len(a[0]) != len(b[0])):
        raise ValueError("The dimension of two matrix is not same.")
    else:
        newMatrix = []
        for i in range(len(a)):
            newMatrix.append([])
            for j in range(len(a[0])):
                newMatrix[i].append(a[i][j] + b[i][j])
        return newMatrix
    
#check the matrix is square matrix or not
def isSquare(a: list) -> bool:
    if(len(a) == len(a[0]) or len(a) == 1):
        return True
    return False

#help the multiplication function to add values
def sumOfArray(a: list, b: list, n: int) -> float:
    sum = 0
    for i in range(len(a)):
        sum += a[i] * b[i][n]
    return sum

#transpose the matrix
def transpose(a: list) -> list:
    newMatrix = []
    for i in range(len(a[0])):
        newMatrix.append([])
        for j in range(len(a)):
            newMatrix[i].append(a[j][i])
    return newMatrix
    
def multiplication(a: list, b: list) -> list:
    """
    i, j
    [[x, x, x],       [[x, x, x],    [[x, x, x],
     [x, x, x],   *    [x, x, x], =   [x, x, x],
     [x, x, x],        [x, x, x]]     [x, x, x],
     [x, x, x]]                       [x, x, x],]
     
    """
    if(len(a[0]) != len(b)):
        raise ValueError("The number of column or the first matrix is not equal to the number of row of the second matrix.")
    else:
        newMatrix = []
        for i in range(len(a)):
            newMatrix.append([])
            for j in range(len(b[0])):
                newMatrix[i].append(sumOfArray(a[i], b, j))
        return newMatrix
    
def getCofactor(a: list, p: int, q: int) -> float:
    temp = []
    for i in range(len(a)):
        if(i != p):
            temp.append([])
        for j in range(len(a[0])):
            if(i != p and j != q):
                temp[len(temp) - 1].append(a[i][j])
    return (-1)**(p + q) * det(temp)

def det(a: list) -> float:
    if(isSquare(a)):
        if(len(a) == 1):
            return a[0]
        elif(len(a) == 2):
            return a[0][0] * a[1][1] - a[0][1] * a[1][0]
        else:
            sum = 0
            for i in range(len(a[0])):
                sum += a[0][i] * getCofactor(a, 0, i)
            return sum
    
def adjoint(a: list) -> list:
    newMatrix = []
    for i in range(len(a)):
        newMatrix.append([])
        for j in range(len(a[0])):
            newMatrix[i].append(getCofactor(a, i, j))
    return transpose(newMatrix)

def inverse(a: list) -> list:
    newMatrix = adjoint(a)
    if(det(a) == 0):
        raise ValueError("The determinant of the matrix is 0, which the matrix is singular.")
    for i in range(len(a)):
        for j in range(len(a[0])):
            newMatrix[i][j] = 1 / det(a) * newMatrix[i][j]
    return newMatrix

a = [[1, 2, 3],
     [4, 5, 6],
     [7, 8, 9]]
b = [[1],
     [2],
     [3]]
print(multiplication(a, b))


from .vector import(
    magnitude,
    dot,
    cross,
    angle,
    projection
)
from .matrix import(
    add,
    det,
    inverse,
    adjoint,
    multiplication,
    transpose
)
from .util import(
    scale,
    printMatrix,
    printVector,
    randomMatrix,
    randomVector,
    solveSystem,
)

__all__ = [
    "add",
    "det",
    "inverse",
    "adjoint",
    "multiplication",
    "transpose",
    "magnitude",
    "dot",
    "cross",
    "angle",
    "projection",
    "scale",
    "printMatrix",
    "printVector",
    "randomMatrix",
    "randomVector",
    "solveSystem"
]


from matrix import *
from vector import *
import random

def scale(a: list, k: float) -> list:
    for i in range(len(a)):
        for j in range(len(a[0])):
            a[i][j] =  k * a[i][j]
    return a

def printMatrix(a: list) -> None:
    for i in range(len(a)):
        if(i != 0):
            print()
        for j in a[i]:
            print(j, end=" ")
    print()

def printVector(a: list) -> None:
    print(str(a[0][0]) + "i + " + str(a[0][1]) + "j + " + str(a[0][2]) + "k")
    
def randomMatrix(row:int, col: int, min: int, max: int) -> list:
    newMatrix = []
    for i in range(row):
        newMatrix.append([])
        for j in range(col):
            newMatrix[i].append(random.randint(min, max))
    return newMatrix

def randomVector(min: int, max: int) -> list:
    return randomMatrix(1, 3, min, max)

def solveSystem(a: list, b: list) -> list:
    if (det(a) == 0):
        return None
    return multiplication(inverse(a), b)
